unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    PollTime: TTimer;
    Panel1: TPanel;
    LogMemo: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    ComCEdit: TEdit;
    ComDEdit: TEdit;
    Label3: TLabel;
    CtPassLabel: TLabel;
    Label4: TLabel;
    CtFailLabel: TLabel;
    Label5: TLabel;
    DelayEdit: TEdit;
    StartStopButton: TButton;
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  Stat00 = MS_RLSD_ON;
  Stat01 = MS_DSR_ON;
  Stat10 = MS_CTS_ON;
  Stat11 = MS_RING_ON;

type
  TRelayState = (rsRelay0, rsRelay1);
  TRelayStates = set of TRelayState;

var
  XState: LongInt;
  ErrCt, PassCt, FailCt: LongInt;
  hComC: THandle = INVALID_HANDLE_VALUE;
  hComD: THandle = INVALID_HANDLE_VALUE;
  TicksPerSec, LastPollTicks: Int64;

function OpenCom(ComStr: AnsiString): THandle;
begin
  ComStr := '\\.\' + ComStr;
  Result := CreateFile(PChar(ComStr), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  ComStr := '';
  if Result = 0 then Result := INVALID_HANDLE_VALUE;
end;

function ModemStatToStr(ModemStat: LongWord): AnsiString;
begin
  Result := '';
  if (ModemStat and MS_RLSD_ON) <> 0 then Result := Result + ' DCD';
  if (ModemStat and MS_DSR_ON ) <> 0 then Result := Result + ' DSR';
  if (ModemStat and MS_CTS_ON ) <> 0 then Result := Result + ' CTS';
  if (ModemStat and MS_RING_ON) <> 0 then Result := Result + ' RI';
  Result := TrimLeft(Result);
end;

function MakeTimestamp: AnsiString;
var
  ST: TSystemTime;

begin
  GetLocalTime(ST);
  with ST do Result := Format('%d:%02d:%02d::%d:%02d:%02d', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);
end;

function RelayStatesFromExpectedState: TRelayStates;
var
  XD: LongInt;

begin
  XD := (XState + 4) mod 8;

  Result := [];
  if (XState mod 5) < 2 then
    Include(Result, rsRelay0)
  ;
  if (XD mod 5) < 2 then
    Include(Result, rsRelay1)
  ;
end;

procedure TForm1.StartButtonClick(Sender: TObject);
var
  ErrC, ErrD: LongWord;

begin
  XState := -1;
  ErrCt := 0;
  PassCt := 0;
  FailCt := 0;
  QueryPerformanceCounter(LastPollTicks);
  LastPollTicks := LastPollTicks - TicksPerSec;

  PollTime.Interval := StrToInt(DelayEdit.Text);

  hComC := OpenCom(ComCEdit.Text);
  ErrC := GetLastError;
  hComD := OpenCom(ComDEdit.Text);
  ErrD := GetLastError;
  if (hComC = INVALID_HANDLE_VALUE) or (hComD = INVALID_HANDLE_VALUE) then begin
    CloseHandle(hComC);
    CloseHandle(hComD);
    if hComC = INVALID_HANDLE_VALUE then LogMemo.Lines.Add('Error ' + IntToStr(ErrC) + ' opening UART C.');
    if hComD = INVALID_HANDLE_VALUE then LogMemo.Lines.Add('Error ' + IntToStr(ErrD) + ' opening UART D.');
    hComC := INVALID_HANDLE_VALUE;
    hComD := INVALID_HANDLE_VALUE;
    Exit;
  end;

  EscapeCommFunction(hComD, SETDTR);

  PollTime.Enabled := True;

  ComCEdit.Enabled := False;
  ComDEdit.Enabled := False;
  DelayEdit.Enabled := False;

  LogMemo.Lines.Add(IntToStr(PollTime.Interval) + 'ms burnin started at ' + MakeTimestamp + '.');

  StartStopButton.Caption := 'Stop';
  StartStopButton.OnClick := StopButtonClick;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  ComCEdit.Enabled := True;
  ComDEdit.Enabled := True;
  DelayEdit.Enabled := True;

  PollTime.Enabled := False;

  CloseHandle(hComC);
  CloseHandle(hComD);

  LogMemo.Lines.Add('Burnin stopped at ' + MakeTimestamp + '.');

  StartStopButton.Caption := 'Start';
  StartStopButton.OnClick := StartButtonClick;
end;

procedure TForm1.PollTimeTimer(Sender: TObject);
var
  ModemStat, XM: LongWord;
  RS: TRelayStates;
  ThisPollTicks: Int64;
  dT: LongWord;

begin
  if not PollTime.Enabled then Exit;

  QueryPerformanceCounter(ThisPollTicks);
  dT := Round(1000/TicksPerSec * (ThisPollTicks - LastPollTicks));
  if dT < PollTime.Interval then Exit;
  LastPollTicks := ThisPollTicks;

  if not GetCommModemStatus(hComD, ModemStat) then begin
    LogMemo.Lines.Add('Error ' + IntToStr(GetLastError) + ' from GetCommModemStatus.');
    Inc(ErrCt);
    if ErrCt > 32 then begin
      LogMemo.Lines.Add('Too many errors, stopping...');
      StopButtonClick(StartStopButton);
    end;
    Exit;
  end;

  ModemStat := ModemStat and (MS_RLSD_ON or MS_DSR_ON or MS_CTS_ON or MS_RING_ON);

  if XState >= 0 then begin
    RS := RelayStatesFromExpectedState;

    XM := 0;
    if rsRelay0 in RS then
      XM := XM or Stat01
    else
      XM := XM or Stat00
    ;
    if rsRelay1 in RS then
      XM := XM or Stat11
    else
      XM := XM or Stat10
    ;

    if ModemStat <> XM then begin
      LogMemo.Lines.Add('Expected ' + ModemStatToStr(XM) + ', got ' + ModemStatToStr(ModemStat) + '.');
      Inc(FailCt);
      if FailCt < 0 then begin
        LogMemo.Lines.Add('Too many failures to count, stopping...');
        StopButtonClick(StartStopButton);
        Exit;
      end;
      CtFailLabel.Caption := IntToStr(FailCt);
    end
    else begin
      Inc(PassCt);
      if FailCt < 0 then begin
        LogMemo.Lines.Add('Too many passes to count, stopping...');
        StopButtonClick(StartStopButton);
        Exit;
      end;
      CtPassLabel.Caption := IntToStr(PassCt);
    end;
  end;

  XState := (XState + 1) mod 8;

  RS := RelayStatesFromExpectedState;
  if rsRelay0 in RS then
    EscapeCommFunction(hComC, SETDTR)
  else
    EscapeCommFunction(hComC, CLRDTR)
  ;
  if rsRelay1 in RS then
    EscapeCommFunction(hComC, SETRTS)
  else
    EscapeCommFunction(hComC, CLRRTS)
  ;

  EscapeCommFunction(hComD, SETRTS);
  EscapeCommFunction(hComD, CLRRTS);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  QueryPerformanceFrequency(TicksPerSec);
  ComCEdit.SelStart := 3;
  ComDEdit.SelStart := 3;
end;

end.
