unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    LogPanel: TPanel;
    ReadTime: TTimer;
    ComRadiosA: TRadioGroup;
    ComRadiosB: TRadioGroup;
    ComRadiosC: TRadioGroup;
    ComRadiosD: TRadioGroup;
    GoButton: TButton;
    LogMemo: TMemo;
    DebugButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure ReadTimeTimer(Sender: TObject);
    procedure DebugButtonClick(Sender: TObject);
  protected
    function IIROTest(IIForRO0, IIForRO1: LongWord): Boolean;
    function DetectSerFail(ToComN: AnsiChar; GotStr: AnsiString; FromComN: AnsiString; ExpectedStr: AnsiString): Boolean;
    procedure UpdateDebugIfNeeded;

    procedure InterCOMAReadCOM(Data: String);
    procedure InterCOMBReadCOM(Data: String);
    procedure InterCOMCReadCOM(Data: String);
    procedure InterCOMDReadCOM(Data: String);
    { Private declarations }
  public
    procedure UserRelaySet(iRelay: Integer; bClosed: Boolean);
    { Public declarations }
  end;

var
  MainForm: TMainForm;


  
implementation

{$R *.DFM}

uses
  InterCOMs, DebugUnit;

const
  RelayInputDelay = 5; //Milliseconds.
  SerialDelay = 32; //Milliseconds.
  SerialBaud = 115200;

var
  hCom: array['A'..'D'] of THandle = (INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE);
  ComRadios: array['A'..'D'] of TRadioGroup;
  MyInterCOM: array['A'..'D'] of TInterCOM;
  InLin: array['A'..'D'] of AnsiString;
  ReadErr, IIData, ROData, NextROData: LongWord;
  iTest: Integer;
  bFail, b0011Fail, b0101Fail, b232Fail, b422Fail, b485Fail: Boolean;
  bInTimer: Boolean;

function OpenCom(ComStr: AnsiString): THandle;
begin
  ComStr := '\\.\' + ComStr;
  Result := CreateFile(PChar(ComStr), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  ComStr := '';
  if Result = 0 then Result := INVALID_HANDLE_VALUE;
end;

procedure RelayEnable(bEnable: Boolean);
begin
  if bEnable then
    EscapeCommFunction(hCom['D'], SETDTR)
  else
    EscapeCommFunction(hCom['D'], CLRDTR)
  ;
end;

procedure RelayUpdate;
begin
  // pulse WR to assert new relay values
  EscapeCommFunction(hCom['D'], CLRRTS);
  EscapeCommFunction(hCom['D'], SETRTS);
  ROData := NextROData;
end;

procedure RelayLoad(iRelay: Integer; bClosed: Boolean);
const
  RelayCmd: array[0..1, Boolean] of Integer = (
    (SETDTR, CLRDTR), //Relay 0 is DTR of UART C.
    (SETRTS, CLRRTS)  //Relay 1 is RTS of UART C.
  );

begin
  EscapeCommFunction(hCom['C'], RelayCmd[iRelay, bClosed]);
  if bClosed then
    NextROData := NextROData or (1 shl iRelay)
  else
    NextROData := NextROData and not (1 shl iRelay)
  ;
end;

procedure IIRead;
var
  ModemStat: LongWord;

begin
  if GetCommModemStatus(hCom['D'], ModemStat) then begin
    ReadErr := ERROR_SUCCESS;
    IIData := 0;
    if (ModemStat and MS_RLSD_ON) <> 0 then IIData := IIData or 1;
    if (ModemStat and MS_DSR_ON ) <> 0 then IIData := IIData or 2;
    if (ModemStat and MS_CTS_ON ) <> 0 then IIData := IIData or 4;
    if (ModemStat and MS_RING_ON) <> 0 then IIData := IIData or 8;
  end
  else begin
    ReadErr := GetLastError;
    if ReadErr = ERROR_SUCCESS then ReadErr := ERROR_GEN_FAILURE;
  end;
end;

function BuildDataBuf(StartCh: AnsiChar): AnsiString;
var
  ThisCh: AnsiChar;

begin
  Result := '';
  ThisCh := StartCh;
  repeat
    Result := Result + ThisCh;
    ThisCh := AnsiChar((Ord(ThisCh) + 1) and $7F);
  until ThisCh = StartCh;
end;



function IntToBin(I, MinBits: LongWord): String;
var
  DoneBits: LongWord;

begin
  Result := '';
  DoneBits := 0;
  repeat
    Result := Char(Byte('0') + (I and 1)) + Result;
    I := I shr 1;
    Inc(DoneBits);
  until (I = 0) and (DoneBits >= MinBits);
end;

function NybbleToHex(N: Byte): Char;
begin
  N := N and $F;
  if N < 10 then
    Result := Char(Byte('0') + N)
  else
    Result := Char(Byte('A') - 10 + N)
  ;
end;

function StrToSpacedHex(Text: String): String;
var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex(Byte(Text[I]) shr 4);
    Result[I * 3 - 1] := NybbleToHex(Byte(Text[I])      );
  end;
end;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

function AnsiMessageBox(hWnd: HWND; Text, Caption: AnsiString; uType: UINT): Integer; overload;
begin
  Result := MessageBox(hWnd, PChar(Text), PChar(Caption), uType);
  Text := '';
  Caption := '';
end;

function AnsiMessageBox(Text, Caption: AnsiString; uType: UINT): Integer; overload;
begin
  Result := AnsiMessageBox(Application.Handle, Text, Caption, uType);
end;



procedure TMainForm.FormCreate(Sender: TObject);
var
  iCom: Integer;
  ComItems: AnsiString;
  hComTemp: THandle;
  ComN: AnsiChar;

begin
  //UARTEditC.SelStart := 3;
  //UARTEditD.SelStart := 3;
  ComRadios['A'] := ComRadiosA;
  ComRadios['B'] := ComRadiosB;
  ComRadios['C'] := ComRadiosC;
  ComRadios['D'] := ComRadiosD;

  ComItems := '';
  for iCom := 1 to 255 do begin
    hComTemp := OpenCom('COM' + IntToStr(iCom));
    if GetLastError <> ERROR_FILE_NOT_FOUND then
      ComItems := ComItems + 'COM' + IntToStr(iCom) + #13#10
    ;
    CloseHandle(hComTemp);
  end;
  for ComN := Low(ComRadios) to High(ComRadios) do
    with ComRadios[ComN] do begin
      Items.Text := ComItems;
      iCom := Ord(ComN) - Ord('A');
      if Items.Count > iCom then ItemIndex := iCom;
    end
  ;
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  ComErr: array['A'..'D'] of LongWord;
  ComN: AnsiChar;
  bOK: Boolean;

begin
  for ComN := Low(ComRadios) to High(ComRadios) do
    CloseHandle(hCom[ComN])
  ;

  bOK := True;
  for ComN := Low(ComRadios) to High(ComRadios) do begin
    InLin[ComN] := '';
    with ComRadios[ComN] do
      hCom[ComN] := OpenCom(Items[ItemIndex])
    ;
    ComErr[ComN] := GetLastError;
    if hCom[ComN] = INVALID_HANDLE_VALUE then bOK := False;
  end;
  if not bOK then begin
    for ComN := Low(ComRadios) to High(ComRadios) do begin
      CloseHandle(hCom[ComN]);
      if hCom[ComN] = INVALID_HANDLE_VALUE then LogMemo.Lines.Add('Error ' + IntToStr(ComErr[ComN]) + ' opening UART ' + ComN + '.');
      hCom[ComN] := INVALID_HANDLE_VALUE;
    end;
    Exit;
  end;

  RelayEnable(True);
  iTest := -1;
  bFail := False;
  b0011Fail := False;
  b0101Fail := False;
  b232Fail := False;
  ReadTime.Enabled := True;
end;



procedure TMainForm.ReadTimeTimer(Sender: TObject);
var
  NextTest: Integer;
  bSucc: Boolean;
  ComN: AnsiChar;

begin
  if not ReadTime.Enabled then Exit;
  if bInTimer then Exit;
  bInTimer := True;
  NextTest := iTest + 1;
  
  case iTest of
    -1: if AnsiMessageBox('Please connect inputs 0 and 1 through relay 0 to power, and inputs 2 and 3 through relay 1 to power.', '0011 II/RO Cables', MB_ICONQUESTION or MB_OKCANCEL)  = IDCANCEL then begin
      LogMemo.Lines.Add('II/RO test cancelled by user.');
      NextTest := 4;
    end;
    0: b0011Fail := IIROTest($3, $C);
    1: if AnsiMessageBox('Please connect inputs 0 and 2 through relay 0 to power, and inputs 1 and 3 through relay 1 to power.', '0101 II/RO Cables', MB_ICONQUESTION or MB_OKCANCEL)  = IDCANCEL then begin
      LogMemo.Lines.Add('II/RO test cancelled by user.');
      NextTest := 4;
    end;
    2: b0101Fail := IIROTest($5, $A);
    3: if b0011Fail or b0101Fail then
      LogMemo.Lines.Add('II/RO Test: FAIL')
    else
      LogMemo.Lines.Add('II/RO Test: pass')
    ;
    4: for ComN := Low(ComRadios) to High(ComRadios) do begin
      CloseHandle(hCom[ComN]);
      hCom[ComN] := INVALID_HANDLE_VALUE;
    end;

    5: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        MyInterCOM[ComN] := TInterCOM.Create(Self);
        MyInterCOM[ComN].ReadAsLines := False;
      end;
      MyInterCOM['A'].OnReadCOM := InterCOMAReadCOM;
      MyInterCOM['B'].OnReadCOM := InterCOMBReadCOM;
      MyInterCOM['C'].OnReadCOM := InterCOMCReadCOM;
      MyInterCOM['D'].OnReadCOM := InterCOMDReadCOM;
    end;

    6: if AnsiMessageBox('Please jumper the device for RS-232, and connect the RS-232 null modem/loopback cables(A to B and C to itself).', 'RS-232 Cables', MB_ICONQUESTION or MB_OKCANCEL) = IDCANCEL then begin
      LogMemo.Lines.Add('232 test cancelled by user.');
      b232Fail := True;
      bFail := True;
      NextTest := 12;
    end;
    7: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        with ComRadios[ComN] do bSucc := MyInterCOM[ComN].Open(StrToInt(Copy(Items[ItemIndex], 4, MAXINT)), SerialBaud);
        if not bSucc then begin
          LogMemo.Lines.Add('Failed to open UART ' + ComN + ' in 232.');
          b232Fail := True;
          bFail := True;
        end;
      end;
      if b232Fail then
        NextTest := 11
      ;
    end;
    8: for ComN := Low(MyInterCOM) to High(MyInterCOM) do
      if InLin[ComN] <> '' then begin
        LogMemo.Lines.Add('UART ' + ComN + ' got spurious data on open: "' + StrToSpacedHex(InLin[ComN]) + '".');
        b232Fail := True;
        bFail := True;
        InLin[ComN] := '';
      end
    ;
    9: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        MyInterCOM[ComN].WriteCOM(BuildDataBuf(ComN))
      ;
      Sleep(SerialDelay);
    end;
    10: begin
      if DetectSerFail('A', InLin['A'], 'B', BuildDataBuf('B')) then
        b232Fail := True
      ;
      if DetectSerFail('B', InLin['B'], 'A', BuildDataBuf('A')) then
        b232Fail := True
      ;
      if DetectSerFail('C', InLin['C'], 'C', BuildDataBuf('C')) then
        b232Fail := True
      ;
      if InLin['D'] <> '' then begin
        b232Fail := True;
        bFail := True;
        LogMemo.Lines.Add('UART D got spurious data. Looped back to itself?');
        LogMemo.Lines.Add('Got ' + IntToStr(Length(InLin['D'])) + ' Bytes: ' + StrToSpacedHex(InLin['D']));
      end;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        InLin[ComN] := ''
      ;
    end;
    11: begin
      if b232Fail then
        LogMemo.Lines.Add('232 Test: FAIL')
      else
        LogMemo.Lines.Add('232 Test: pass')
      ;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        MyInterCOM[ComN].Close
      ;
    end;

    12: if AnsiMessageBox('Please jumper the device for RS-422, and connect the RS-422 "null modem" cables(A to B and A to C).', 'RS-422 Cables', MB_ICONQUESTION or MB_OKCANCEL) = IDCANCEL then begin
      LogMemo.Lines.Add('422 test cancelled by user.');
      b422Fail := True;
      bFail := True;
      NextTest := 18;
    end;
    13: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        with ComRadios[ComN] do bSucc := MyInterCOM[ComN].Open(StrToInt(Copy(Items[ItemIndex], 4, MAXINT)), SerialBaud);
        if not bSucc then begin
          LogMemo.Lines.Add('Failed to open UART ' + ComN + ' in 422.');
          b422Fail := True;
          bFail := True;
        end;
      end;
      if b422Fail then
        NextTest := 17
      ;
    end;
    14: for ComN := Low(MyInterCOM) to High(MyInterCOM) do
      if InLin[ComN] <> '' then begin
        LogMemo.Lines.Add('UART ' + ComN + ' got spurious data on open: "' + StrToSpacedHex(InLin[ComN]) + '".');
        b422Fail := True;
        bFail := True;
        InLin[ComN] := '';
      end
    ;
    15: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        MyInterCOM[ComN].WriteCOM(BuildDataBuf(ComN));
        Sleep(SerialDelay);
      end;
    end;
    16: begin
      if DetectSerFail('A', InLin['A'], 'B & C', BuildDataBuf('B') + BuildDataBuf('C')) then
        b422Fail := True
      ;
      if DetectSerFail('B', InLin['B'], 'A', BuildDataBuf('A')) then
        b422Fail := True
      ;
      if DetectSerFail('C', InLin['C'], 'A', BuildDataBuf('A')) then
        b422Fail := True
      ;
      if InLin['D'] <> '' then begin
        b422Fail := True;
        bFail := True;
        LogMemo.Lines.Add('UART D got spurious data. Looped back to itself?');
        LogMemo.Lines.Add('Got ' + IntToStr(Length(InLin['D'])) + ' Bytes: ' + StrToSpacedHex(InLin['D']));
      end;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        InLin[ComN] := ''
      ;
    end;
    17: begin
      if b422Fail then
        LogMemo.Lines.Add('422 Test: FAIL')
      else
        LogMemo.Lines.Add('422 Test: pass')
      ;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        MyInterCOM[ComN].Close
      ;
    end;

    18: if AnsiMessageBox('Please jumper the device for RS-485, and connect the RS-485 "null modem" cables(A to B to C, party line).', 'RS-485 Cables', MB_ICONQUESTION or MB_OKCANCEL) = IDCANCEL then begin
      LogMemo.Lines.Add('485 test cancelled by user.');
      b485Fail := True;
      bFail := True;
      NextTest := 24;
    end;
    19: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        with ComRadios[ComN] do bSucc := MyInterCOM[ComN].Open(StrToInt(Copy(Items[ItemIndex], 4, MAXINT)), SerialBaud);
        if not bSucc then begin
          LogMemo.Lines.Add('Failed to open UART ' + ComN + ' in 485.');
          b485Fail := True;
          bFail := True;
        end;
      end;
      if b485Fail then
        NextTest := 23
      ;
    end;
    20: for ComN := Low(MyInterCOM) to High(MyInterCOM) do
      if InLin[ComN] <> '' then begin
        LogMemo.Lines.Add('UART ' + ComN + ' got spurious data on open: "' + StrToSpacedHex(InLin[ComN]) + '".');
        b485Fail := True;
        bFail := True;
        InLin[ComN] := '';
      end
    ;
    21: begin
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do begin
        MyInterCOM[ComN].WriteCOM(BuildDataBuf(ComN));
        Sleep(SerialDelay);
      end;
    end;
    22: begin
      if DetectSerFail('A', InLin['A'], 'B & C', BuildDataBuf('B') + BuildDataBuf('C')) then
        b485Fail := True
      ;
      if DetectSerFail('B', InLin['B'], 'A & C', BuildDataBuf('A') + BuildDataBuf('C')) then
        b485Fail := True
      ;
      if DetectSerFail('C', InLin['C'], 'A & B', BuildDataBuf('A') + BuildDataBuf('B')) then
        b485Fail := True
      ;
      if InLin['D'] <> '' then begin
        b485Fail := True;
        bFail := True;
        LogMemo.Lines.Add('UART D got spurious data. Looped back to itself?');
        LogMemo.Lines.Add('Got ' + IntToStr(Length(InLin['D'])) + ' Bytes: ' + StrToSpacedHex(InLin['D']));
      end;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        InLin[ComN] := ''
      ;
    end;
    23: begin
      if b485Fail then
        LogMemo.Lines.Add('485 Test: FAIL')
      else
        LogMemo.Lines.Add('485 Test: pass')
      ;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do
        MyInterCOM[ComN].Close
      ;
    end;

    24: begin
      ReadTime.Enabled := False;
      if bFail then
        LogMemo.Lines.Add('Overall: FAIL')
      else
        LogMemo.Lines.Add('Overall: pass')
      ;
      for ComN := Low(MyInterCOM) to High(MyInterCOM) do 
        MyInterCOM[ComN].Free
      ;
    end;
  end;

  iTest := NextTest;
  bInTimer := False;
end;

function TMainForm.IIROTest(IIForRO0, IIForRO1: LongWord): Boolean;
var
  I: Integer;
  Expected: LongWord;

begin
  Result := False;
  LogMemo.Lines.Add('Testing relays and inputs, ' + IntToBin(IIForRO1, 4) + '...');
  for I := 0 to 32 do begin
    RelayLoad(0, ((I and 1) <> 0) xor ((I and 2) <> 0));
    RelayLoad(1, (I and 4) <> 0);
    RelayUpdate;
    Expected := 0;
    if (ROData and 1) <> 0 then Expected := Expected or IIForRO0;
    if (ROData and 2) <> 0 then Expected := Expected or IIForRO1;

    Sleep(RelayInputDelay);

    IIRead;
    if IIData <> Expected then begin
      LogMemo.Lines.Add('Mismatch: Expected ' + IntToBin(Expected, 4) + ', got ' + IntToBin(IIData, 4) + '.');
      Result := True;
      bFail := True;
    end;
  end;
end;

function TMainForm.DetectSerFail(ToComN: AnsiChar; GotStr: AnsiString; FromComN: AnsiString; ExpectedStr: AnsiString): Boolean;
begin
  Result := GotStr <> ExpectedStr;
  if Result then begin
    bFail := True;
    LogMemo.Lines.Add('UART ' + ToComN + ' did not get expected data from UART ' + FromComN + '.');
    LogMemo.Lines.Add('Expected ' + IntToStr(Length(ExpectedStr)) + ' Bytes: ' + StrToSpacedHex(ExpectedStr));
    LogMemo.Lines.Add('Got ' + IntToStr(Length(GotStr)) + ' Bytes: ' + StrToSpacedHex(GotStr));
  end;
end;



procedure TMainForm.UpdateDebugIfNeeded;
begin
  if DebugForm = nil then Exit;

  DebugForm.NoteIIROData(ReadErr, IIData, ROData);
end;

procedure TMainForm.DebugButtonClick(Sender: TObject);
begin
  if DebugForm = nil then
    Application.CreateForm(TDebugForm, DebugForm)
  ;
  UpdateDebugIfNeeded;
  DebugForm.Show;
end;

procedure TMainForm.UserRelaySet(iRelay: Integer; bClosed: Boolean);
begin
  RelayLoad(iRelay, bClosed);
  RelayUpdate;
end;

procedure TMainForm.InterCOMAReadCOM(Data: String);
begin
  InLin['A'] := InLin['A'] + Data;
end;

procedure TMainForm.InterCOMBReadCOM(Data: String);
begin
  InLin['B'] := InLin['B'] + Data;
end;

procedure TMainForm.InterCOMCReadCOM(Data: String);
begin
  InLin['C'] := InLin['C'] + Data;
end;

procedure TMainForm.InterCOMDReadCOM(Data: String);
begin
  InLin['D'] := InLin['D'] + Data;
end;

end.
