unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    MainPanel: TPanel;
    ReadTime: TTimer;
    ReadCheck: TCheckBox;
    Label1: TLabel;
    UARTEditC: TEdit;
    Label2: TLabel;
    UARTEditD: TEdit;
    InLabel0: TLabel;
    Label4: TLabel;
    InLabel1: TLabel;
    Label3: TLabel;
    InLabel2: TLabel;
    Label5: TLabel;
    InLabel3: TLabel;
    Label6: TLabel;
    OutButton0: TSpeedButton;
    Label8: TLabel;
    OutButton1: TSpeedButton;
    Label9: TLabel;
    OutButton2: TSpeedButton;
    Label10: TLabel;
    OutButton3: TSpeedButton;
    Label11: TLabel;
    GoButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ReadCheckClick(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure OutButton0Click(Sender: TObject);
    procedure OutButton1Click(Sender: TObject);
    procedure OutButton2Click(Sender: TObject);
    procedure OutButton3Click(Sender: TObject);
    procedure ReadTimeTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

var
  hComC: THandle = INVALID_HANDLE_VALUE;
  hComD: THandle = INVALID_HANDLE_VALUE;

function OpenCom(ComStr: AnsiString): THandle;
begin
  ComStr := '\\.\' + ComStr;
  Result := CreateFile(PChar(ComStr), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  ComStr := '';
  if Result = 0 then Result := INVALID_HANDLE_VALUE;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  UARTEditC.SelStart := 3;
  UARTEditD.SelStart := 3;
end;

procedure TMainForm.ReadCheckClick(Sender: TObject);
begin
  ReadTime.Enabled := ReadCheck.Checked;
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  ErrC, ErrD: LongWord;

begin
  CloseHandle(hComC);
  CloseHandle(hComD);
  ReadCheck.Checked := False;
  MainPanel.Visible := False;
  Caption := 'ComSwitch';

  hComC := OpenCom(UARTEditC.Text);
  ErrC := GetLastError;
  hComD := OpenCom(UARTEditD.Text);
  ErrD := GetLastError;
  if (hComC = INVALID_HANDLE_VALUE) or (hComD = INVALID_HANDLE_VALUE) then begin
    CloseHandle(hComC);
    CloseHandle(hComD);
    if hComC = INVALID_HANDLE_VALUE then Caption := Caption + ', error ' + IntToStr(ErrC) + ' opening UART C';
    if hComD = INVALID_HANDLE_VALUE then Caption := Caption + ', error ' + IntToStr(ErrD) + ' opening UART D';
    hComC := INVALID_HANDLE_VALUE;
    hComD := INVALID_HANDLE_VALUE;
    Exit;
  end;
  EscapeCommFunction(hComD, SETDTR); // ENABLE RELAYS
  OutButton2.Down := True;
  
  MainPanel.Visible := True;
end;

procedure RelayWrite;
begin
    // pulse WR to assert new Relay value
    EscapeCommFunction(hComD, CLRRTS);
    EscapeCommFunction(hComD, SETRTS);
end;

procedure TMainForm.OutButton0Click(Sender: TObject);
begin
  if OutButton0.Down then
    EscapeCommFunction(hComC, SETDTR)
  else
    EscapeCommFunction(hComC, CLRDTR)
  ;
  RelayWrite;
end;

procedure TMainForm.OutButton1Click(Sender: TObject);
begin
  if OutButton1.Down then
    EscapeCommFunction(hComC, SETRTS)
  else
    EscapeCommFunction(hComC, CLRRTS)
  ;
  RelayWrite;
end;

procedure TMainForm.OutButton2Click(Sender: TObject);
begin
  if OutButton2.Down then
    EscapeCommFunction(hComD, SETDTR)
  else
    EscapeCommFunction(hComD, CLRDTR)
  ;
end;

procedure TMainForm.OutButton3Click(Sender: TObject);
begin
  // level control over WR pulse
  if OutButton3.Down then
    EscapeCommFunction(hComD, SETRTS)
  else
    EscapeCommFunction(hComD, CLRRTS)
  ;
  // RelayWrite; // pulse
end;

function IBoolToColor(IBool: LongWord): TColor;
begin
  if IBool <> 0 then
    Result := clRed
  else
    Result := clBlack
  ;
end;

procedure TMainForm.ReadTimeTimer(Sender: TObject);
var
  ModemStat: LongWord;
  ErrStr: AnsiString;

begin
  if GetCommModemStatus(hComD, ModemStat) then begin
    InLabel0.Color := IBoolToColor(ModemStat and MS_RLSD_ON);
    InLabel1.Color := IBoolToColor(ModemStat and MS_DSR_ON);
    InLabel2.Color := IBoolToColor(ModemStat and MS_CTS_ON);
    InLabel3.Color := IBoolToColor(ModemStat and MS_RING_ON);
    InLabel0.ShowHint := False;
    InLabel1.ShowHint := False;
    InLabel2.ShowHint := False;
    InLabel3.ShowHint := False;
  end
  else begin
    ErrStr := 'Error ' + IntToStr(GetLastError) + ' from GetCommModemStatus.';
    InLabel0.ParentColor := True;
    InLabel1.ParentColor := True;
    InLabel2.ParentColor := True;
    InLabel3.ParentColor := True;
    InLabel0.Hint := ErrStr;
    InLabel0.ShowHint := True;
    InLabel1.Hint := ErrStr;
    InLabel1.ShowHint := True;
    InLabel2.Hint := ErrStr;
    InLabel2.ShowHint := True;
    InLabel3.Hint := ErrStr;
    InLabel3.ShowHint := True;
  end;
  OutButton1.Down := not OutButton1.Down;
  OutButton1Click(self);
end;

end.
